import logging

from gpiozero import Button

from carom.config import config
from carom.utils import singleton

logger = logging.getLogger(__name__)


@singleton
class Table:
    def __init__(self, num, port):
        self.num = num
        self.port = port
        self.button = Button(port, pull_up=False)

    def __repr__(self):
        return "<Table num={} port={} in_use={}>".format(self.num, self.port, self.in_use)

    def __del__(self):
        self.close()

    @property
    def in_use(self):
        return self.button.is_active

    def close(self):
        logger.debug("Closing GPIO {}".format(self.port))
        self.button.close()


def cleanup():
    for table in get_tables():
        table.close()


def _populate_tables():
    if config is None:
        return []
    for num, port in config['mapping'].items():
        yield Table(num, port)


def get_tables():
    return [table for table in _populate_tables()]


__all__ = ('Table', 'get_tables', 'cleanup')
